﻿#region Copyright © 2014 Miracle Business Solutions
// All rights are reserved. Reproduction or transmission in whole or in part,
// any form or by any means, electronic, mechanical or otherwise, is prohibited
// without the prior written consent of the copyright owner.
// 
// Document:	AutoFocus\AutoFocus.cs
// Creation Date:	24/11/2013
// Last Edit:		28/03/2014
// Author:		Moreno Sint Hill
#endregion

namespace AutoFocus
{
    using System;
    using System.IO;
    using System.Linq;
    using System.Windows.Forms;
    using System.Windows.Media;
    using Styx;
    using Styx.Common;
    using Styx.Helpers;
    using Styx.Plugins;
    using Styx.TreeSharp;
    using Styx.WoWInternals;
    using Styx.WoWInternals.WoWObjects;
    using Action = Styx.TreeSharp.Action;

    public class AutoFocus : HBPlugin
    {
        // root of our plugins behavior tree
        private Composite Root;

        #region Overrides of HBPlugin

        // standard plugin overrides
        private static bool _rescan;
        private Form _newtempui;

        public override string Name
        {
            get { return "AutoFocus - A "; }
        }

        public override string Author
        {
            get { return "Mirabis"; }
        }

        public override Version Version
        {
            get { return new Version(2, 0, 0, 0); }
        }

        public override bool WantButton
        {
            get { return true; }
        }

        /// <summary>
        ///     Initialize() equals Enabled
        ///     ---
        ///     called when user clicks the enable checkbox for the plugin, -or when
        ///     HonorBuddy loads plugins initially if already initialized
        /// </summary>
        public override void Initialize()
        {
            base.Initialize();
            _rescan = true;
            Logging.Write(Colors.Pink, "[AutoFocus] Enabled");
            Lua.Events.AttachEvent("PARTY_MEMBERS_CHANGED", Handler);
            // create behavior tree and Start (prepares to run when pulsed)
            Root = CreateBehaviorLogic();
            Root.Start(null);
        }

        /// <summary>
        ///     PartySize Changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        private void Handler(object sender, LuaEventArgs args)
        {
            _rescan = true;
            Logging.Write(Colors.Pink, "[AutoFocus] Detected change in party, queuing a new scan");
        }

        /// <summary>
        ///     Called when the button for this CC is pressed.
        /// </summary>
        public override void OnButtonPress()
        {
            //Settings.Instance.Load();
            if (_newtempui == null || _newtempui.IsDisposed || _newtempui.Disposing) _newtempui = new GUI();
            if (_newtempui != null || _newtempui.IsDisposed) _newtempui.ShowDialog();
        }

        /// <summary>
        ///     Dispose() equals Disabled
        ///     ---
        ///     called when user unchecks your plugin -or- when HonorBuddy is closing.  Any
        ///     resource, hook, or event you implemented in Initialize() must be released here
        ///     to avoid the side-effects of your plugin continuing to run after the user disables
        /// </summary>
        public override void Dispose()
        {
            Lua.Events.DetachEvent("PARTY_MEMBERS_CHANGED", Handler);
            Logging.Write(Colors.Pink, "[AutoFocus] Disabled");
            if (StyxWoW.Me.FocusedUnitGuid == AFSettings.Instance.FocusGUID)
                Lua.DoString("RunMacroText('/clearfocus')");
            // stop the behavior tree and release
            Root.Stop(null);
            Root = null;

            base.Dispose();
        }

        /// <summary>
        ///     Pulse() equals Behavior Tree Tick()
        ///     ---
        ///     This will Tick your behavior tree, so does not need to be modified.
        ///     Implement your plugins logic in the CreateBehaviorLogic() method.
        ///     ---
        ///     This style of behavior tree plugin will cause your behavior to tick
        ///     Pulse.  An alternative is to hook one of the predefined TreeHooks
        ///     so your tree runs only during the associated state (Rest, Combat, etc.)
        ///     For an example of tying to a predefined TreeHook see:
        ///     http://www.thebuddyforum.com/honorbuddy-forum/community-developer-forum/114540-answers-treesharp-questions.html#post1114517
        /// </summary>
        public override void Pulse()
        {
            try
            {
                Root.Tick(null);
                // If the last status wasn't running, stop the tree, and restart it.
                if (Root.LastStatus != RunStatus.Running)
                {
                    Root.Stop(null);
                    Root.Start(null);
                }
            }
            catch (Exception e)
            {
                // Restart on any exception.
                Logging.WriteException(e);
                Root.Stop(null);
                Root.Start(null);
                throw;
            }
        }

        #endregion

        #region Plugin Logic

        /// <summary>
        ///     Plugin Logic Goes Here
        ///     --
        ///     This creates the Behavior Tree which represents your plugins logic.
        /// </summary>
        private static Composite CreateBehaviorLogic()
        {
// .. logic for behavior tree goes here ..
            return
                new Decorator(
                    req =>
                        _rescan || StyxWoW.Me.FocusedUnit == null ||
                        StyxWoW.Me.FocusedUnitGuid != AFSettings.Instance.FocusGUID,
                    new Switch<TargetType>(ctx => AFSettings.Instance.SelectedType,
                        new SwitchArgument<TargetType>(TargetType.Arena1,
                            new Decorator(req => StyxWoW.Me.IsInArena,
                                new Action(ctx =>
                                {
                                    ulong newguid = StyxWoW.Me.GetPartyMemberGuid(1);
                                    AFSettings.Instance.FocusGUID = newguid;
                                    StyxWoW.Me.SetFocus(newguid);
                                    _rescan = false;
                                }))),
                        new SwitchArgument<TargetType>(TargetType.Arena2,
                            new Decorator(req => StyxWoW.Me.IsInArena,
                                new Action(ctx =>
                                {
                                    ulong newguid = StyxWoW.Me.GetPartyMemberGuid(2);
                                    AFSettings.Instance.FocusGUID = newguid;
                                    StyxWoW.Me.SetFocus(newguid);
                                    _rescan = false;
                                }))),
                        new SwitchArgument<TargetType>(TargetType.Arena3,
                            new Decorator(req => StyxWoW.Me.IsInArena,
                                new Action(ctx =>
                                {
                                    ulong newguid = StyxWoW.Me.GetPartyMemberGuid(3);
                                    AFSettings.Instance.FocusGUID = newguid;
                                    StyxWoW.Me.SetFocus(newguid);
                                    _rescan = false;
                                }))),
                        new SwitchArgument<TargetType>(TargetType.Healer,
                            new Decorator(req => StyxWoW.Me.GroupInfo.IsInParty,
                                new Action(ctx =>
                                {
                                    WoWPartyMember healer =
                                        StyxWoW.Me.GroupInfo.PartyMembers.FirstOrDefault(
                                            a => a.Role == WoWPartyMember.GroupRole.Healer && a.IsOnline);

                                    if (healer != null)
                                    {
                                        Logging.Write(Colors.Pink, "[AutoFocus] Focusing {0} as it's our Healer",
                                            healer.ToPlayer().SafeName);

                                        AFSettings.Instance.FocusGUID = healer.Guid;
                                        StyxWoW.Me.SetFocus(healer.Guid);
                                    }
                                    _rescan = false;
                                }))),
                        new SwitchArgument<TargetType>(TargetType.Tank,
                            new Decorator(req => StyxWoW.Me.GroupInfo.IsInParty,
                                new Action(ctx =>
                                {
                                    WoWPartyMember tank =
                                        StyxWoW.Me.GroupInfo.PartyMembers.FirstOrDefault(
                                            a => a.Role == WoWPartyMember.GroupRole.Tank && a.IsOnline);

                                    if (tank != null)
                                    {
                                        Logging.Write(Colors.Pink, "[AutoFocus] Focusing {0} as it's our Tank",
                                            tank.ToPlayer().SafeName);

                                        AFSettings.Instance.FocusGUID = tank.Guid;
                                        StyxWoW.Me.SetFocus(tank.Guid);
                                    }
                                    _rescan = false;
                                }))),
                        new SwitchArgument<TargetType>(TargetType.TargetByGuid,
                            new Action(ctx =>
                            {
                                Logging.Write(Colors.Pink, "[AutoFocus] Lost focus , focusing {0} as configured",
                                    AFSettings.Instance.FocusName);
                                StyxWoW.Me.SetFocus(AFSettings.Instance.FocusGUID);
                                _rescan = false;
                            }))));
        }

        #endregion

        internal enum TargetType
        {
            Tank = 0,
            Healer,
            Arena1,
            Arena2,
            Arena3,
            TargetByGuid
        }

        #region Settings

        internal class AFSettings : Settings
        {
            private static AFSettings _instance;

            public AFSettings()
                : base(
                    Path.Combine(Path.Combine(Utilities.AssemblyDirectory, "Settings"),
                        string.Format("AFSettings{0}.xml", StyxWoW.Me.Name)))
            {
            }

            public static AFSettings Instance
            {
                get { return _instance ?? (_instance = new AFSettings()); }
            }

            [Setting]
            public TargetType SelectedType { get; set; }

            [Setting]
            public string FocusName { get; set; }

            [Setting]
            public ulong FocusGUID { get; set; }
        }

        #endregion
    }
}