﻿#region Copyright © 2013 Miracle Business Solutions
// All rights are reserved. Reproduction or transmission in whole or in part,
// any form or by any means, electronic, mechanical or otherwise, is prohibited
// without the prior written consent of the copyright owner.
// 
// Document:	AutoFocus\GUI.cs
// Creation Date:	24/11/2013
// Last Edit:		24/11/2013
// Author:		Moreno Sint Hill
#endregion

using System;
using System.Drawing;
using System.Windows.Forms;
using Styx.Common;
using Styx.Helpers;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

namespace AutoFocus
{
    public partial class GUI : Form
    {
        public GUI()
        {
            Styx.AutoFocus.AFSettings.Instance.Load();
            InitializeComponent();
            SetComboBoxEnum(TargetBox,(int) Styx.AutoFocus.AFSettings.Instance.SelectedType);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            //Set Name
           Styx.AutoFocus.AFSettings.Instance.FocusName = tbName.Text;
            //Search
            var unit = ObjectManager.GetObjectsOfTypeFast<WoWPlayer>()
                    .Find(a => a.Name.Equals(Styx.AutoFocus.AFSettings.Instance.FocusName));

            if (unit != null && unit.IsValid)
            {
                Styx.AutoFocus.AFSettings.Instance.FocusGUID = unit.Guid;
                tbGUID.Text = "Succeeded";
                tbGUID.ForeColor = Color.Green;
            }
            else
            {
                tbGUID.Text = "Failed";
                tbGUID.ForeColor = Color.Red;
            }
        }

        public void PopulateComboBox()
        {
            TargetBox.Items.Add(new CboItem((int)Styx.AutoFocus.TargetType.Tank, "Tank"));
            TargetBox.Items.Add(new CboItem((int)Styx.AutoFocus.TargetType.Healer, "Healer"));
            TargetBox.Items.Add(new CboItem((int)Styx.AutoFocus.TargetType.TargetByGuid, "TargetByGuid"));
            TargetBox.Items.Add(new CboItem((int)Styx.AutoFocus.TargetType.Dps1, "Dps 1"));
            TargetBox.Items.Add(new CboItem((int)Styx.AutoFocus.TargetType.Dps2, "Dps 2"));
            TargetBox.Items.Add(new CboItem((int)Styx.AutoFocus.TargetType.Dps3, "Dps 3"));
            TargetBox.Items.Add(new CboItem((int)Styx.AutoFocus.TargetType.Arena1, "Arena 1"));
            TargetBox.Items.Add(new CboItem((int)Styx.AutoFocus.TargetType.Arena2, "Arena 2"));
            TargetBox.Items.Add(new CboItem((int)Styx.AutoFocus.TargetType.Arena3, "Arena 3"));
        }   

        private static void SetComboBoxEnum(ComboBox cb, int e)
        {
            try
            {
                CboItem item;
                for (int i = 0; i < cb.Items.Count; i++)
                {
                    item = (CboItem)cb.Items[i];
                    if (item.E != e) continue;
                    cb.SelectedIndex = i;
                    return;
                }
                item = (CboItem)cb.Items[0];
                if (item != null)
                {
                    Logging.Write("Dialog Error: combobox {0} does not have enum({1}) in list, defaulting to enum({2})",
                   cb.Name, e, item.E);
                }
                else Logging.Write("Dialog Error: combobox {0} does not have enum({1}) in list, defaulting to enum(null)",
                 cb.Name, e);
                cb.SelectedIndex = 0;
            }
            catch (Exception ex)
            {
             Logging.WriteException(ex.InnerException);
            }

        }
        
        private static int GetComboBoxEnum(ComboBox cb)
        {
            try
            {
                var item = (CboItem)cb.Items[cb.SelectedIndex];
                return item.E;
            }
            catch (Exception ex)
            {
                Logging.WriteException(ex.InnerException);
            }

            return 0;
        }

        internal class CboItem
        {
            public readonly int E;
            private readonly string _s;

            public CboItem(int pe, string ps)
            {
                E = pe;
                _s = ps;
            }

            public override string ToString()
            {
                return _s;
            }
        }

        private void TargetTypeChanged(object sender, EventArgs e)
        {
            Styx.AutoFocus.AFSettings.Instance.SelectedType = (Styx.AutoFocus.TargetType) GetComboBoxEnum(TargetBox);
        }
    }
}
